@@ -84,12 +84,9 @@ void LLVMToSPIRVDbgTran::transDebugMetadata() {
 SPIRVValue *
 LLVMToSPIRVDbgTran::createDebugDeclarePlaceholder(const DbgDeclareInst *DbgDecl,
                                                   SPIRVBasicBlock *BB) {
-  if (!DbgDecl->getAddress())
-    return nullptr; // The variable is dead.
-
   DbgDeclareIntrinsics.push_back(DbgDecl);
   using namespace SPIRVDebug::Operand::DebugDeclare;
-  SPIRVWordVec Ops(OperandCount, getDebugInfoNone()->getId());
+  SPIRVWordVec Ops(OperandCount, getDebugInfoNoneId());
   SPIRVId ExtSetId = BM->getExtInstSetId(SPIRVEIS_Debug);
   return BM->addExtInst(getVoidTy(), ExtSetId, SPIRVDebug::Declare, Ops, BB);
 }
@@ -108,7 +105,8 @@ void LLVMToSPIRVDbgTran::finalizeDebugDeclare(const DbgDeclareInst *DbgDecl) {
   using namespace SPIRVDebug::Operand::DebugDeclare;
   SPIRVWordVec Ops(OperandCount);
   Ops[DebugLocalVarIdx] = transDbgEntry(DbgDecl->getVariable())->getId();
-  Ops[VariableIdx] = SPIRVWriter->transValue(Alloca, BB)->getId();
+  Ops[VariableIdx] = Alloca ? SPIRVWriter->transValue(Alloca, BB)->getId()
+                            : getDebugInfoNoneId();
   Ops[ExpressionIdx] = transDbgEntry(DbgDecl->getExpression())->getId();
   DD->setArguments(Ops);
 }
@@ -352,6 +350,18 @@ SPIRVEntry *LLVMToSPIRVDbgTran::getScope(DIScope *S) {
   }
 }
 
+SPIRVEntry *LLVMToSPIRVDbgTran::getGlobalVariable(const DIGlobalVariable *GV) {
+  for (GlobalVariable &V : M->globals()) {
+    SmallVector<DIGlobalVariableExpression *, 4> GVs;
+    V.getDebugInfo(GVs);
+    for (DIGlobalVariableExpression *GVE : GVs) {
+      if (GVE->getVariable() == GV)
+        return SPIRVWriter->transValue(&V, nullptr);
+    }
+  }
+  return getDebugInfoNone();
+}
+
 SPIRVWord mapDebugFlags(DINode::DIFlags DFlags) {
   SPIRVWord Flags = 0;
   if ((DFlags & DINode::FlagAccessibility) == DINode::FlagPublic)
@@ -451,7 +461,9 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgBaseType(const DIBasicType *BT) {
   ConstantInt *Size = getUInt(M, BT->getSizeInBits());
   Ops[SizeIdx] = SPIRVWriter->transValue(Size, nullptr)->getId();
   auto Encoding = static_cast<dwarf::TypeKind>(BT->getEncoding());
-  Ops[EncodingIdx] = SPIRV::DbgEncodingMap::map(Encoding);
+  SPIRVDebug::EncodingTag EncTag = SPIRVDebug::Unspecified;
+  SPIRV::DbgEncodingMap::find(Encoding, &EncTag);
+  Ops[EncodingIdx] = EncTag;
   return BM->addDebugInfo(SPIRVDebug::TypeBasic, getVoidTy(), Ops);
 }
 
@@ -756,20 +768,15 @@ LLVMToSPIRVDbgTran::transDbgGlobalVariable(const DIGlobalVariable *GV) {
   if (Context && (isa<DINamespace>(Context) || isa<DISubprogram>(Context)))
     Parent = transDbgEntry(Context);
   Ops[ParentIdx] = Parent->getId();
+
   Ops[LinkageNameIdx] = BM->getString(GV->getLinkageName())->getId();
-
-  // Variable
-  SPIRVEntry *Var = getDebugInfoNone();
-  llvm::GlobalVariable *V = M->getGlobalVariable(GV->getName());
-  if (!V)
-    V = M->getGlobalVariable(GV->getLinkageName());
-  if (V)
-    Var = SPIRVWriter->transValue(V, nullptr);
-  Ops[VariableIdx] = Var->getId();
-
+  Ops[VariableIdx] = getGlobalVariable(GV)->getId();
   Ops[FlagsIdx] = transDebugFlags(GV);
+
+  // Check if GV is the definition of previously declared static member
   if (DIDerivedType *StaticMember = GV->getStaticDataMemberDeclaration())
     Ops.push_back(transDbgEntry(StaticMember)->getId());
+
   return BM->addDebugInfo(SPIRVDebug::GlobalVariable, getVoidTy(), Ops);
 }
 
@@ -786,7 +793,11 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgFunction(const DISubprogram *Func) {
   Ops[SourceIdx] = getSource(Func)->getId();
   Ops[LineIdx] = Func->getLine();
   Ops[ColumnIdx] = 0; // This version of DISubprogram has no column number
-  Ops[ParentIdx] = getScope(Func->getScope())->getId();
+  auto Scope = Func->getScope();
+  if (Scope && isa<DIFile>(Scope))
+    Ops[ParentIdx] = SPIRVCU->getId();
+  else
+    Ops[ParentIdx] = getScope(Scope)->getId();
   Ops[LinkageNameIdx] = BM->getString(Func->getLinkageName())->getId();
   Ops[FlagsIdx] = transDebugFlags(Func);
 
@@ -799,15 +810,14 @@ SPIRVEntry *LLVMToSPIRVDbgTran::transDbgFunction(const DISubprogram *Func) {
     Ops.resize(MinOperandCount);
     Ops[ScopeLineIdx] = Func->getScopeLine();
 
-    llvm::Function *F = M->getFunction(Func->getName());
-    if (!F)
-      F = M->getFunction(Func->getLinkageName());
-    if (F) {
-      SPIRVValue *SPIRVFunc = SPIRVWriter->getTranslatedValue(F);
-      assert(SPIRVFunc && "All function must be already translated");
-      Ops[FunctionIdIdx] = SPIRVFunc->getId();
-    } else {
-      Ops[FunctionIdIdx] = getDebugInfoNoneId();
+    Ops[FunctionIdIdx] = getDebugInfoNoneId();
+    for (const llvm::Function &F : M->functions()) {
+      if (Func->describes(&F)) {
+        SPIRVValue *SPIRVFunc = SPIRVWriter->getTranslatedValue(&F);
+        assert(SPIRVFunc && "All function must be already translated");
+        Ops[FunctionIdIdx] = SPIRVFunc->getId();
+        break;
+      }
     }
 
     if (DISubprogram *FuncDecl = Func->getDeclaration())
